# _*_ coding: iso-8859-1 _*_
#!/usr/bin/env python

from twisted.spread import pb
from twisted.internet import threads, reactor

import win32api as wa
import ntsecuritycon as ntsec
import win32security as wsec

import platform
import _winreg
import sys
import os
import socket
import time

from logging.handlers import RotatingFileHandler
import logging

from killProcName import killProcName
from pbgest import PBgest
from messagebox import gmessage
from lance_cmd import lancecmd
from lecteur import map
from perms import set_acl, AdjustPrivilege
from reg import get_option


class Logon:
    def __init__(self, user, domaine):
        self.username = user.strip().lower()
        self.domname = domaine.strip().lower()
        self.scribe_dom = get_option(r'Software\Eole\Scribe', 'nom_domaine').lower()
        self.scribe_name = get_option(r'Software\Eole\Scribe', 'nom_scribe').lower()
        ost = platform.release().lower()
        if 'xp' in ost:
            self.type_os = 'WinXP'
        elif '2000' in ost:
            self.type_os = 'Win2K'
        elif '2003' in ost:
            self.type_os = 'Win2K3'
        elif 'nt' in ost:
            self.type_os = 'WinNT'
        else:
            self.type_os = 'Win95'
        fich_ext = '.txt'
        self.userfich = '%s%s%s'%(self.username, self.type_os, fich_ext) #userWinXP.txt
        self.fpath = r'\\%s\netlogon\%s'%(self.scribe_name, self.userfich)
        # privilges de modifier des ACLs, fichier de log si debug=on, dossier temporaire
        AdjustPrivilege([ntsec.SE_SECURITY_NAME, ntsec.SE_RESTORE_NAME])
        self.init_log()
        self.init_tmpdir()
    
    def init_log(self):
        """active (si possible) le fichier de log si HKLM\Eole\Scribe : debug_logon est renseign  "1"
        """
        self.log = None
        if get_option(r'Software\Eole\Scribe', 'debug_logon') == '1':
            try:
                # on log sur 2 fichiers de 100ko
                logfile = os.path.join(os.environ['WINDIR'], 'cliscribe_logon.log')
                try: set_acl(logfile, self.domname)
                except: pass
                handler = RotatingFileHandler(logfile,'a',100000,2)
                _formatter = logging.Formatter('%(asctime)s (%(process)d) %(levelname)s %(message)s')
                handler.setFormatter(_formatter)
                self.log = logging.getLogger(logfile)
                self.log.addHandler(handler)
                self.log.setLevel(logging.INFO)
            except: pass
    
    def init_tmpdir(self):
        """Immdiatement aprs le login, TEMPDIR pointe ailleurs que Docu&Settings\<user>\Local Settings\Temp
        a serait plutt WINDIR\Temp
        Le cre si besoin et pose des ACLs pour DomainUsers
        """
        try:
            tmpdir = wa.GetTempPath()
            self.log_event('TMPDIR = %s'%tmpdir)
            if not os.path.exists(tmpdir): os.makedirs(tmpdir)
            set_acl(tmpdir, self.domname)
            esu_fond_ecran = os.path.join(tmpdir, '_ESU_LOGIN.BMP')
            if not os.path.exists(esu_fond_ecran): file(esu_fond_ecran, 'w').write('')
            set_acl(esu_fond_ecran, self.domname)
        except: pass

    def test_logon_type(self):
        """S'agit-il d'un login du domaine
        ou d'un login local
        """
        self.log_event('domaine : %s, dom_scribe : %s'%(self.domname, self.scribe_dom))
        if self.domname == self.scribe_dom: return True
        else: return False
    
    def logon(self):
        """Cherche le fichier <user><Os>.txt le parcour et le traite
        Si OK, appel "remote_logon" sur controle-vnc-serveur
        """
        self.log_event('Ouverture de session de %s sur %s'%(self.username, self.domname))
        if self.test_logon_type():
            if not os.path.isfile(self.fpath):
                self.return_err('Fichier "%s" non trouv'%self.fpath)
            if self.traitefich():
                # appel de la mthode remote_logon avec type_os en argument
                cmd = os.path.join(get_option(r'Software\Eole\Scribe', 'inst_path'), 'cliscribe', 'utilisateur.exe')
                lancecmd(cmd, nowait=True)
                s = socket.socket()
                cport = get_option(r'Software\Eole\Scribe', 'port_utilisateur')
                while True:
                    try:
                        time.sleep(0.5)
                        s.connect(('127.0.0.1', int(cport)))
                        s.close()
                        break
                    except Exception, e:
                        self.log_event('Passage %s'%e)
                        pass
                self.log_event('Appel de remote_logon(%s)'%self.type_os)
                self.pbgest = PBgest()
                self.pbgest.d.addCallback(self.return_pbroot)
                self.pbgest.d.addErrback(self.return_err)
            else:
                reactor.stop()
        else:
            self.log_event('Abandon')
            reactor.stop()
    
    def traitefich(self):
        """Lit le fichier <user><Os>.txt
        et le traite ligne par ligne
        """
        self.log_event('Traitement de %s'%self.fpath)
        for ligne in [ i.strip() for i in file(self.fpath, 'r').readlines() ]:
            if len(ligne) > 0:
                if not self.parse_line(ligne): return False
        return True

    def parse_line(self, line):
        """Traite une ligne du fichier
        cmd = commande  lancer, options possibles :
            - HIDDEN : cache la fentre d'excution
            - NOWAIT : n'attend pas le retour
        lecteur = lettre et partage  monter dessus :
            si lecteur existe dj et est un lecteur rseau, il est pralablement dmont
            sinon passage au lecteur suivant
        """
        try:
            items = line.split(',')
            # excuter une commande
            if items[0] == 'cmd':
                hide, nowait = False, False
                if 'HIDDEN' in items: hide = True
                if 'NOWAIT' in items: nowait = True
                self.log_event('cmd : %s HIDDEN=%s NOWAIT=%s'%(items[1], hide, nowait))
                ret = lancecmd(items[1], hide, nowait)
                if ret != 0:
                    self.gmessage = 'La commande "%s" a choue avec le code de sortie "%s"'%(items[1], ret)
                    a = threads.deferToThread(self.mbox)
                    return False
                else: return True
            # monter un partage
            if items[0] == 'lecteur':
                lettre, partage = items[1], items[2]
                self.log_event('lecteur : %s %s'%(lettre, partage))
                ret = map(lettre, partage)
                if ret[0]:
                    if ret[1] != lettre:
                        self.gmessage = 'Le lecteur "%s" tait occupe, utilisation de "%s" pour "%s"'%(lettre, ret[1], partage)
                        a = threads.deferToThread(self.mbox)
                else:
                    self.gmessage = 'Impossible de monter le partage %s'%(ret[1])
                    a = threads.deferToThread(self.mbox)
                return ret[0]
        except Exception, e:
            self.gmessage = '%s : %s'%(line, e)
            a = threads.deferToThread(self.mbox)
            return False

    def return_pbroot(self, pbroot):
        """Fonction de retour du CallBack de GetRootObject
        """
        self.pbroot = pbroot
        d = self.pbroot.callRemote('logon', self.type_os)
        d.addCallback(self.return_logon)
        d.addErrback(self.return_err)

    def return_logon(self, ret=''):
        """Fonction de retour du CallBack de CallRemote("remote_logon")
        relance explorer.exe si dj lanc
        (TODO amliorer la relance, on se retrouve avec l'explorateur de fichiers lanc en plus ...)
        """
        self.gmessage = 'Retour remote_logon : %s'%ret
        self.log_event(self.gmessage)
        reactor.stop()

    def return_err(self, o='', msg=''):
        """Fonction d'erreur, peut servir  un ErrBack
        """
        try: self.gmessage = '%s'%o.value
        except: self.gmessage = o
        self.log_event('Erreur : %s'%self.gmessage)
        a = threads.deferToThread(self.mbox)
        try: reactor.stop()
        except: pass

    def log_event(self, e):
        """Enregistre l'vnement "e" dans le fichier de log si debug=on
        """
        if self.log:
            try: self.log.info(e)
            except: pass

    def mbox(self):
        gmessage(self.gmessage, "Gestion d'ouverture de sessions")

        
def lance():
    try:
        user, domain = sys.argv[1:3]
    except:
        user = wa.GetUserName()
        domain = wa.GetDomainName()
    s = Logon(user, domain)
    threads.deferToThread(s.logon)

def main():
    lance()
    reactor.run()
    
if __name__ == '__main__':
    main()
